#! c:\perl\bin\perl.exe
#---------------------------------------------------------
# keytime.pl
# Retrieves LastWrite time from Registry keys
#
# usage: keytime.pl <full key path>
# ex:    keytime.pl HKEY_LOCAL_MACHINE\Software\Microsoft
#
# Requires Win32::TieRegistry
# Requires Win32::API::Prototype
# ppm install http://www.roth.net/perl/packages/win32-api-prototype.ppd
#---------------------------------------------------------
use strict;
use Win32::API::Prototype;
use Win32::TieRegistry(Delimiter=>"/");

my @month = qw/Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec/;
my @day = qw/Sun Mon Tue Wed Thu Fri Sat/;
my %tags = ("HKLM" => "HKEY_LOCAL_MACHINE",
            "HKCU" => "HKEY_CURRENT_USER");

my $server = Win32::NodeName;
my $regkey = shift || die "You must enter a Registry key.\n";
$regkey = "HKEY_LOCAL_MACHINE\\".$regkey if ($regkey =~ m/^software/i || $regkey =~ m/^system/i);
foreach my $tag (keys %tags) {
	$regkey =~ s/$tag/$tags{$tag}/ if ($regkey =~ m/^$tag/);
}

$regkey =~ s/\\/\//g;
my %info;
my $remote;
# Registry key to check
eval {
	$remote = $Registry->{"//$server/".$regkey};
	die "Could not locate $regkey.\n" unless ($remote);
};
print $@."\n" if ($@);
die "Key has no Information.\n" unless (%info = $remote->Information);

ApiLink('kernel32.dll', 
        'BOOL  FileTimeToLocalFileTime(FILETIME *lpFileTime, 
                             LPFILETIME lpLocalFileTime )' ) 
    || die "Can not locate FileTimeToLocalFileTime()";

ApiLink('kernel32.dll', 
        'BOOL  FileTimeToSystemTime(FILETIME *lpFileTime, 
                             LPSYSTEMTIME lpSystemTime )' ) 
    || die "Can not locate FileTimeToSystemTime()";

my $pFileTime = $info{'LastWrite'};
# Credit goes to Ian Brown <ibrown@mitchellandmitchell.com> with locating
# a buffer overflow issue b/c $lpLocalFileTime was not set up initially
my $lpLocalFileTime = pack ("L2", 0);
# Create an empty SYSTEMTIME structure of 8 short ints
# pack()'d together
my $pSystemTime = pack("S8", 0);

# Translate the FILETIME to LOCALFILETIME
if (FileTimeToLocalFileTime($pFileTime,$lpLocalFileTime)) {

# call FileTimeToSystemTime()
	if (FileTimeToSystemTime($lpLocalFileTime,$pSystemTime)) {

# Unpack the 8 WORD values from the system time structure....
# year,month,dayofweek,day,hour,minute,sec,milli
  	my @time = unpack("S8", $pSystemTime);
  
  	$time[5] = "0".$time[5] if ($time[5] =~ m/^\d$/);
  	$time[6] = "0".$time[6] if ($time[6] =~ m/^\d$/);
  	my $timestr = $day[$time[2]]." ".$time[1]."/".$time[3]."/".$time[0].
                " ".$time[4].":".$time[5].":".$time[6].":".$time[7];
  	print "$timestr\n";
  	print "\n";
	}
}